Attribute VB_Name = "mod_Draw_Balloon_s4p_230715"
Option Compare Database
Option Explicit

'*************** Code Start *****************************************************
' module name: mod_Draw_Balloon_s4p
'-------------------------------------------------------------------------------
' Purpose  : VBA to draw a balloon on an Access report
'              send report object, center coordinate, and size
'              optionally colors, text, and more
' Author   : crystal (strive4peace)
' Code List: www.MsAccessGurus.com/code.htm
' This code: https://msaccessgurus.com/VBA/Draw_BirthdayBalloons.htm
' LICENSE  :
'   You may freely use and share this code, but not sell it.
'   Keep attribution. Use at your own risk.
'-------------------------------------------------------------------------------
'used by GetRandomInteger, not balloon
Private mbRandomize As Boolean
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'           Draw_Balloon_s4p
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Sub Draw_Balloon_s4p(oReport As Report _
   , xCenter As Double _
   , yCenter As Double _
   , dbRadius As Double _
   , Optional pnColor As Long = vbYellow _
   , Optional pnBorderColor As Long = -1 _
   , Optional psText As String = "" _
   , Optional piFontSize As Integer = 10 _
   , Optional piFontColor As Long = 16777215 _
   )
'220617 strive4peace, 230630
'draw a filled balloon (Aspect=1.2) with pnColor
'  slightly offset a black shadow
'  draw a string
'psText is made smaller than piFontSize if it won't fit
'
'
   'PARAMETERS
   '  oReport is the Report object
   '  pnColor is the color for the fill. Default is black
   '  pnBorderColor will be pnColor if not specified
   '  psText is text to write in the middle
   '  piFontSize is (starting) font size to use for text
   '  piFontColor is color for text, default is white

   On Error GoTo Proc_Err
      
   Dim dbAspect As Double _
      , x1 As Double, y1 As Double _
      , x2 As Double, y2 As Double _
      , i As Integer _
      , iFontSize As Integer _
      , iShadowOffset As Integer

   iShadowOffset = 40
   
   With oReport
      
      .ScaleMode = 1 'twips
      .DrawWidth = 1 'pixel
            
      .FillStyle = 0 'Opaque
      
      If pnBorderColor < 0 Then
         pnBorderColor = pnColor
      End If
      'oval shaped balloon
      dbAspect = 1.2
      
      '.Circle (x,y), Radius, Color, StartAngle, EndAngle, Aspect
      'balloon black shadow
      .FillColor = 0
      oReport.Circle (xCenter + iShadowOffset _
         , yCenter + iShadowOffset) _
         , dbRadius _
         , 0, , , dbAspect
      'balloon
      .FillColor = pnColor
      oReport.Circle (xCenter, yCenter) _
         , dbRadius _
         , pnBorderColor, , , dbAspect
      
      If psText <> "" Then
         .ForeColor = piFontColor
         iFontSize = piFontSize
         .FontSize = iFontSize
         Do While .TextWidth(psText) _
                  > dbRadius * dbAspect
            iFontSize = iFontSize - 1
            .FontSize = iFontSize
         Loop
         
         .CurrentX = xCenter - .TextWidth(psText) / 2
         .CurrentY = yCenter - .TextHeight(psText) / 2
         .Print psText
      End If
      
      'draw bottom   'dbAspect
      x1 = xCenter - dbRadius / 12
      x2 = xCenter + dbRadius / 12
      
      y1 = yCenter + dbRadius
      y2 = yCenter + dbRadius + dbRadius / 16
      
      'shadow
      oReport.Line (x1, y1)-(x2 + iShadowOffset _
         , y2 + iShadowOffset _
         ), 0, BF
      
      oReport.Line (x1, y1)-(x2, y2), pnColor, BF
            
      'draw string
      y1 = y2
      y2 = y1 + dbRadius * 2
      
      oReport.Line (xCenter, y1)-( _
         xCenter, y2) _
         , RGB(200, 200, 200)
      
   End With
      
Proc_Exit:
   Exit Sub
  
Proc_Err:
   MsgBox Err.Description _
       , , "ERROR " & Err.Number _
        & "   Draw_Balloon_s4p"

   Resume Proc_Exit
   Resume
   
End Sub

'===================================================
'  this is needed for example report
'        to position and color balloons
'        , not to draw a balloon
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'           GetRandomInteger
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Function GetRandomInteger(piMinumum As Integer _
   , piMaximum As Integer _
   , Optional pDummy As Variant _
   ) As Integer
's4p 220616, 708, 230715
   'test module variable to only do
   'at beginning of a loop or first record of SQL
   If mbRandomize <> True Then
      Randomize
      mbRandomize = True
   End If
   'Fix instead of Int in case numbers are negative
   GetRandomInteger = _
      Fix( _
         ((piMaximum - piMinumum + 1) _
         * Rnd) _
         + piMinumum)
End Function

'*************** Code End *****************************************************


